import java.io.*;
import java.util.ArrayList;

public class Circuit
{
    String name;
    Node ref;
    ArrayList<Node> nodes;
    ArrayList<Device> devices;
    String [] inputs;
    String [] outputs;

    Circuit()
    {
	ref = new Node("0");
	devices = new ArrayList<Device>();
	nodes = new ArrayList<Node>();
    }

    public void read(BufferedReader inr) {
	LineNumberReader in = new LineNumberReader(inr);
	boolean done = false;
	while (!done) {
	    int n = in.getLineNumber();
	    try {
		String inp = in.readLine();
		if (inp==null) break;
		System.out.println("Line " + n + ":  " + inp);
		String [] str = inp.split("%"); // terminate at comment character
		String [] tokens = str[0].split("\\s+");
		if (tokens.length>1) {
		    // This version just does gates
		    Device d = Gate.getNewDevice(tokens[0]);
		    if (d!=null) {
			d.parse(tokens);
			devices.add(d);
		    }
		    else if (tokens[0].equals("inputs")) inputs = saveTokens(tokens);
		    else if (tokens[0].equals("outputs")) outputs = saveTokens(tokens);
		    else System.out.println("Device not recognized: " + tokens[0] );
		}
	    }
	    catch (IOException e) {
		System.out.println(e);
	    }
	}
	System.out.println();
    }

    String [] saveTokens(String tokens[])
    {
	int i, ntokens;
	ntokens = tokens.length;
	String [] list = new String[ntokens-1];
	for (i=1; i<ntokens; i++) list[i-1] = tokens[i];
	return list;
    }

    Node find_node(String str)
    {
	int ncount = nodes.size();
	for (int j=0; j<ncount; j++) {
	    Node node = nodes.get(j);
	    if (node.name.equals(str)) return node;
	}
	return null;
    }


    int add_node(String str, Device dev)
    {
	int i;
	Node node;
	int ncount = nodes.size();
	for (i=0; i<ncount; i++) {
	    node = nodes.get(i);
	    if (node.name.equals(str)) {
		node.devices.add(dev);
		return i;
	    }
	}
	node = new Node(str);
	nodes.add(node);
	node.devices.add(dev);
	return ncount;
    }

    void build_node_list()
    {
	int i, j;
	Device dev;
	int ndevices = devices.size();
	for (i=0; i<ndevices; i++) {
	    dev = devices.get(i);
	    int nodecount = dev.nodes.length;
	    for (j=0; j<nodecount; j++) dev.ndx[j] = add_node(dev.nodes[j],dev);
	}
    }

    void list_nodes()
    {
	for (Node node: nodes) System.out.println(node);
	/*
	int i;
	int ncount = nodes.size();
	for (i=0; i<ncount; i++) {
	    Node node =  nodes.get(i);
	    System.out.printf("%d\t %s\n",i,node);
	}
	*/
    }
	

    void list_devices()
    {
	for (Device d: devices) System.out.println(d);
    }

    int eval(ArrayList<Node> changed_in, ArrayList<Node> changed_out)
    {
	ArrayList<Gate> devices = new ArrayList<Gate>(0);
	Gate gate;
	for (Node node: changed_in) {
	    for (Device dev: node.devices) {
		gate = (Gate) dev;
		// we only evaluate gates
		if (!dev.getClass().isInstance(gate)) continue;
		if (!devices.contains(gate)) devices.add(gate);
	    }
	}
	for (Gate dev: devices) dev.eval(nodes,changed_out);
	return changed_out.size();
    }
}
