import java.util.ArrayList;

public abstract class Device {
    String name;
    String [] nodes;
    int [] ndx;
    double value;

    Device(int n)
    {
	nodes = new String[n];
	ndx = new int[n];
	value = 0.0;
    }

    String getType() { return "device"; }
    String getCode() { return "device_code"; }

    public void parse(String tok[])
    {
	name = tok[1];
	int j, n, ntokens;
	ntokens = tok.length;
	n = nodes.length;
	if (ntokens < n+2) {
	    System.out.printf("%d tokens %d needed\n",ntokens,n+2);
	    System.err.println("Input error - device has insufficient nodes");
	}
	for (j=0; j<n; j++) nodes[j] = tok[2+j];
	n += 2;
	if (ntokens<=n) return;
	// logic may not have a value, but it might have an orientation
	value = parseValue(tok[n]);	
    }
    
    public String toString()
    {
	String str = getType() + " " + getCode() + " " + name;
	for (String s: nodes) str += " " + s;
	str += " " + value;
	return str;
    }
	
    static public double parseValue(String str)
    {
	final String suffix = "afpnumkKMGT";
	final double scales[] = {1e-18, 1e-15, 1e-12, 1e-9, 1e-6, 1e-3, 1e3, 1e3, 1e6, 1e9, 1e12};
	char [] chars = str.toCharArray();
	int n = chars.length;
	while (n>0) {
	    if (!Character.isLetter(chars[n-1])) break;
	    n--;
	}
	double v;
	try {
	    if (n<chars.length) {
		v = Double.parseDouble(str.substring(0,n));
		//System.out.printf("value %g suffix %s\n",v,str.substring(n));
		n = suffix.indexOf(chars[n]);
		if (n<0) return v;
		else v *= scales[n];
	    }
	    else v = Double.parseDouble(str);
	}
	catch (NumberFormatException e) {
	    System.out.println(e);
	    v = 0.0;
	}
	return v;
    }
}
