import java.util.ArrayList;


public class Gate extends Device
{
    Gate(int n)
    {
	super(n);
    }
    
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed) { return -1; }

    // This version is like Device, except there is no "value" field.
    public String toString()
    {
	String str = getCode() + "\t" + name;
	for (String s: nodes) str += "\t" + s;
	return str;
    }

    public static Device getNewDevice(String type)
    {
	if (type.equals(Inverter.code)) return new Inverter();
	if (type.equals(And.code)) return new And();
	if (type.equals(Or.code)) return new Or();
	if (type.equals(Nand.code)) return new Nand();
	if (type.equals(Nor.code)) return new Nor();
	if (type.equals(Xor.code)) return new Xor();
	return null;
    }

    static int aoi(Node arg[], int in, int out)
    {
	double result = (out>0? 0: 1);
	double vin = in;
	for (int i=1; i<3; i++) {
	    Node node = arg[i];
	    if (node.isUnknown()) result = 0.5;
	    else if (node.equals(vin)) {
		result = out;
		break;
	    }
	}
	if (arg[0].equals(result)) return 0;
	arg[0].setValue(result);
	return 1;
    }
}

class Inverter extends Gate {
    final static String code = "INV";
    Inverter() {
	super(2);
    }
    String getType() { return "inverter"; }
    String getCode() { return Inverter.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node inp = nodes.get(ndx[0]);
 	Node oup = nodes.get(ndx[1]);
	double vout = 0.5;
	if (inp.isLow()) vout = 1.0;
	else if (inp.isHigh()) vout = 0.0;
	if (oup.equals(vout)) return 0;
	oup.setValue(vout);
	changed.add(oup);
	return 1;
    }
}

class And extends Gate {
    final static String code = "AND";
    And() {
	super(3);
    }
    String getType() { return "and"; }
    String getCode() { return And.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node arg[] = new Node[3];
	for (int i=0; i<3; i++) arg[i] = nodes.get(ndx[i]);
	int iret = Gate.aoi(arg,0,0);
	if (iret>0) changed.add(arg[0]);
	return iret;
    }
}


class Or extends Gate {
    final static String code = "OR";
    Or() {
	super(3);
    }
    String getType() { return "or"; }
    String getCode() { return Or.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node arg[] = new Node[3];
	for (int i=0; i<3; i++) arg[i] = nodes.get(ndx[i]);
	int iret = Gate.aoi(arg,1,1);
	if (iret>0) changed.add(arg[0]);
	return iret;
    }
}

class Nand extends Gate {
    final static String code = "NAND";
    Nand() {
	super(3);
    }
    String getType() { return "nand"; }
    String getCode() { return Nand.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node arg[] = new Node[3];
	for (int i=0; i<3; i++) arg[i] = nodes.get(ndx[i]);
	int iret = Gate.aoi(arg,0,1);
	if (iret>0) changed.add(arg[0]);
	return iret;
    }
}

class Nor extends Gate {
    final static String code = "NOR";
    Nor() {
	super(3);
    }
    String getType() { return "nor"; }
    String getCode() { return Nor.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node arg[] = new Node[3];
	for (int i=0; i<3; i++) arg[i] = nodes.get(ndx[i]);
	int iret = Gate.aoi(arg,1,0);
	if (iret>0) changed.add(arg[0]);
	return iret;
    }
}

class Xor extends Gate {
    final static String code = "XOR";
    Xor() {
	super(3);
    }
    String getType() { return "xor"; }
    String getCode() { return Xor.code; }
    int eval(ArrayList<Node> nodes, ArrayList<Node> changed)
    {
	Node arg[] = new Node[3];
	for (int i=0; i<3; i++) arg[i] = nodes.get(ndx[i]);
	double result = 0.5;
	if (arg[1].isLow() && arg[2].isLow()) result = 0.0;
	else if (arg[1].isHigh() && arg[2].isHigh()) result = 0.0;
	else if (arg[1].isUnknown() || arg[2].isUnknown()) result = 0.5;
	else result = 1.0;
	if (arg[0].equals(result)) return 0;
	arg[0].setValue(result);
	changed.add(arg[0]);
	return 1;
    }
}