// gates1.java

import java.io.*;
import java.util.ArrayList;


// This class is a modified version of netlist3.java
public class gates1 {

    static public void main(String args[])
    {
	String filename;
	FileInputStream instream;
	BufferedReader in;
	
	if (args.length < 1) {
	    System.err.println("usage: java gates1 source");
	    System.exit(0);
 	}
	try {
	    filename = args[0];
	    instream = new FileInputStream(filename);
	    in = new BufferedReader(new InputStreamReader(instream));
	    Circuit ckt = new Circuit();
	    ckt.read(in);
	    in.close();
	    System.out.println("\nDevice list:\n");
	    ckt.list_devices();
	    ckt.build_node_list();
	    System.out.println("\nNode list:\n");
	    ckt.list_nodes();
	    if (ckt.inputs!=null || ckt.outputs!= null) System.out.println("\n\n");
	    if (ckt.inputs!=null) show("inputs",ckt.inputs);
	    if (ckt.outputs!=null) show("outputs",ckt.outputs);
	    showTable(ckt);
	} catch (IOException e) {  
		 System.out.println( e);
	}
    }

    static void show(String name, String vals[])
    {
	int j, nvals;
	String str = name;
	for (String val: vals) str += "\t" + val;
	System.out.println(str);
    }

    static void showTable(Circuit ckt)
    {
	String [] inputs = ckt.inputs;
	String [] outputs = ckt.outputs;
	int ninp = inputs.length;
	int noup = outputs.length;
	int ncases = (int) Math.pow(2,ninp);
	int j, k, v;
	String str = "";
	System.out.println("\n");
	for (j=0; j<ninp; j++) {
	    if (j>0) str += "\t" + inputs[j];
	    else str = inputs[j];
	}
	for (j=0; j<noup; j++) str += "\t" + outputs[j];
	System.out.println(str + "\tcycles\tmax_changed");
	
	Node [] input_nodes = new Node[ninp];
	ArrayList<Node> changed_in = new ArrayList<Node>();
	ArrayList<Node> changed_out = new ArrayList<Node>();
	for (j=0; j<ninp; j++) {
	    Node node = ckt.find_node(inputs[j]);
	    if (node==null) System.err.printf("node %s not found\n",inputs[j]);
	    input_nodes[j] = node;
	}

	Node [] output_nodes = new Node[noup];
	for (j=0; j<noup; j++) {
	    Node node = ckt.find_node(outputs[j]);
	    if (node==null) System.err.printf("node %s not found\n",outputs[j]);
	    output_nodes[j] = node;
	}
	// cycle through the possible input values (for a truth table)
	for (k=0; k<ncases; k++) {
	    for (j = 0; j<ninp; j++) {
		v = (k>>(ninp-1-j))&1;
		if (j>0) str += "\t" + v;
		else str = "" + v;
		Node node = input_nodes[j];
		node.setValue(v);
		changed_in.add(node);

	    }

	    // evaluate the circuit until nothing changes
	    int ncycles, change;
	    ncycles = 0;
	    change = ckt.eval(changed_in,changed_out);
	    int max_change = change;	    
	    while (change>0) {
		ncycles++;
		changed_in.clear();
		changed_in.addAll(changed_out);
		changed_out.clear();
		change = ckt.eval(changed_in,changed_out);
		if (change>max_change) max_change = change;
	    }
	    
	    // print the output values
	    for (j=0; j<noup; j++) str += "\t" + output_nodes[j].getLogic();
	    System.out.println(str + "\t" + ncycles + "\t" + max_change);
	}
    }
	
}

		